@@ -370,11 +370,11 @@ class QuicStreamTest(TestCase):
         self.assertEqual(stream.sender.next_offset, 16)
 
         # first chunk gets acknowledged
-        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 0, 8)
+        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 0, 8, False)
         self.assertFalse(stream.sender.is_finished)
 
         # second chunk gets acknowledged
-        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 8, 16)
+        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 8, 16, False)
         self.assertFalse(stream.sender.is_finished)
 
     def test_sender_data_and_fin(self):
@@ -409,11 +409,11 @@ class QuicStreamTest(TestCase):
         self.assertEqual(stream.sender.next_offset, 16)
 
         # first chunk gets acknowledged
-        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 0, 8)
+        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 0, 8, False)
         self.assertFalse(stream.sender.is_finished)
 
         # second chunk gets acknowledged
-        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 8, 16)
+        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 8, 16, True)
         self.assertTrue(stream.sender.is_finished)
 
     def test_sender_data_and_fin_ack_out_of_order(self):
@@ -448,11 +448,11 @@ class QuicStreamTest(TestCase):
         self.assertEqual(stream.sender.next_offset, 16)
 
         # second chunk gets acknowledged
-        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 8, 16)
+        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 8, 16, True)
         self.assertFalse(stream.sender.is_finished)
 
         # first chunk gets acknowledged
-        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 0, 8)
+        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 0, 8, False)
         self.assertTrue(stream.sender.is_finished)
 
     def test_sender_data_lost(self):
@@ -489,7 +489,7 @@ class QuicStreamTest(TestCase):
         self.assertEqual(stream.sender.next_offset, 16)
 
         # a chunk gets lost
-        stream.sender.on_data_delivery(QuicDeliveryState.LOST, 0, 8)
+        stream.sender.on_data_delivery(QuicDeliveryState.LOST, 0, 8, False)
         self.assertEqual(list(stream.sender._pending), [range(0, 8)])
         self.assertEqual(stream.sender.next_offset, 0)
 
@@ -535,7 +535,7 @@ class QuicStreamTest(TestCase):
         self.assertEqual(stream.sender.next_offset, 16)
 
         # a chunk gets lost
-        stream.sender.on_data_delivery(QuicDeliveryState.LOST, 8, 16)
+        stream.sender.on_data_delivery(QuicDeliveryState.LOST, 8, 16, True)
         self.assertEqual(list(stream.sender._pending), [range(8, 16)])
         self.assertEqual(stream.sender.next_offset, 8)
 
@@ -547,8 +547,12 @@ class QuicStreamTest(TestCase):
         self.assertEqual(list(stream.sender._pending), [])
         self.assertEqual(stream.sender.next_offset, 16)
 
-        # both chunks gets acknowledged
-        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 0, 16)
+        # first chunk gets acknowledged
+        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 0, 8, False)
+        self.assertFalse(stream.sender.is_finished)
+
+        # second chunk gets acknowledged
+        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 8, 16, True)
         self.assertTrue(stream.sender.is_finished)
 
     def test_sender_blocked(self):
@@ -635,6 +639,10 @@ class QuicStreamTest(TestCase):
         self.assertIsNone(frame)
         self.assertTrue(stream.sender.buffer_is_empty)
 
+        # EOF is acknowledged
+        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 0, 0, True)
+        self.assertTrue(stream.sender.is_finished)
+
     def test_sender_fin_only_despite_blocked(self):
         stream = QuicStream()
 
@@ -657,6 +665,30 @@ class QuicStreamTest(TestCase):
         self.assertIsNone(frame)
         self.assertTrue(stream.sender.buffer_is_empty)
 
+    def test_sender_fin_then_ack(self):
+        stream = QuicStream()
+
+        # send some data
+        stream.sender.write(b"data")
+        frame = stream.sender.get_frame(8)
+        self.assertEqual(frame.data, b"data")
+
+        # data is acknowledged
+        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 0, 4, False)
+        self.assertFalse(stream.sender.is_finished)
+
+        # write EOF
+        stream.sender.write(b"", end_stream=True)
+        self.assertFalse(stream.sender.buffer_is_empty)
+        frame = stream.sender.get_frame(8)
+        self.assertEqual(frame.data, b"")
+        self.assertTrue(frame.fin)
+        self.assertEqual(frame.offset, 4)
+
+        # EOF is acknowledged
+        stream.sender.on_data_delivery(QuicDeliveryState.ACKED, 4, 4, True)
+        self.assertTrue(stream.sender.is_finished)
+
     def test_sender_reset(self):
         stream = QuicStream()
 
